from pathlib import Path

from PySide6.QtCore import Slot, Qt
from PySide6.QtUiTools import loadUiType
from PySide6.QtWidgets import QMainWindow, QSlider
from vtkmodules.vtkRenderingOpenGL2 import vtkGenericOpenGLRenderWindow

from crystalfly.ui.import_file import ImportFile
from crystalfly.ui.volume_viewer import VolumeViewer
from crystalfly.ui.vtk_viewer import VTKImageViewer

MainWindowType, _ = loadUiType(str(Path(__file__).parent.joinpath("main_window.ui")))


class MainWindow(QMainWindow, MainWindowType):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.volume = None
        self.transverse = None
        self.sagittal = None
        self.coronal = None
        self.vtk_render_window = vtkGenericOpenGLRenderWindow()
        self.setupUi(self)
        self.file_open_dialog = ImportFile()
        self.file_open.triggered.connect(self.handle_file_open)
        self.init_four_pane()
        self.is_pane_maximum = False

    def init_four_pane(self):
        self.coronal = VTKImageViewer(1)
        self.sagittal = VTKImageViewer(3)
        self.transverse = VTKImageViewer(4)
        self.volume = VolumeViewer(2)
        self.coronal.image_viewer.SetSliceOrientationToYZ()
        self.sagittal.image_viewer.SetSliceOrientationToXZ()
        self.transverse.image_viewer.SetSliceOrientationToXY()

        self.volume.doubleClicked.connect(self.cell_max)
        self.coronal.doubleClicked.connect(self.cell_max)
        self.sagittal.doubleClicked.connect(self.cell_max)
        self.transverse.doubleClicked.connect(self.cell_max)

        self.volume.clicked.connect(self.cell_active)
        self.coronal.clicked.connect(self.cell_active)
        self.sagittal.clicked.connect(self.cell_active)
        self.transverse.clicked.connect(self.cell_active)

        self.coronal.range_changed.connect(self.set_slide_range)
        self.sagittal.range_changed.connect(self.set_slide_range)
        self.transverse.range_changed.connect(self.set_slide_range)

        self.volume_grid_layout.addWidget(self.volume)
        self.coronal_grid_layout.addWidget(self.coronal, 0, 0, 8, 8)
        self.gridLayout_3.addWidget(self.sagittal, 0, 0, 8, 8)
        self.gridLayout_4.addWidget(self.transverse, 0, 0, 8, 8)

        self.coronal_slider = QSlider(Qt.Orientation.Vertical)
        self.sagittal_slider = QSlider(Qt.Orientation.Vertical)
        self.transverse_slider = QSlider(Qt.Orientation.Vertical)

        self.coronal_slider.valueChanged.connect(self.coronal.image_viewer.SetSlice)
        self.sagittal_slider.valueChanged.connect(self.sagittal.image_viewer.SetSlice)
        self.transverse_slider.valueChanged.connect(self.transverse.image_viewer.SetSlice)

        self.coronal_grid_layout.addWidget(self.coronal_slider, 0, 8, 8, 1)
        self.gridLayout_3.addWidget(self.sagittal_slider, 0, 8, 8, 1)
        self.gridLayout_4.addWidget(self.transverse_slider, 0, 8, 8, 1)

    @Slot(int, int)
    def set_slide_range(self, quadrant, value):
        if quadrant == 1:
            self.coronal_slider.setRange(0, value)
        elif quadrant == 3:
            self.sagittal_slider.setRange(0, value)
        elif quadrant == 4:
            self.transverse_slider.setRange(0, value)

    @Slot(int)
    def cell_max(self, quadrant: int):
        if self.is_pane_maximum:
            self.widget_3d.show()
            self.widget_coronal.show()
            self.widget_sagittal.show()
            self.widget_transverse.show()
            self.is_pane_maximum = False
        else:
            self.widget_3d.hide()
            self.widget_coronal.hide()
            self.widget_sagittal.hide()
            self.widget_transverse.hide()
            if quadrant == 1:
                self.widget_coronal.show()
            elif quadrant == 2:
                self.widget_3d.show()
            elif quadrant == 3:
                self.widget_sagittal.show()
            elif quadrant == 4:
                self.widget_transverse.show()
            self.is_pane_maximum = True

    @Slot(int)
    def cell_active(self, quadrant: int):
        self.widget_3d.setStyleSheet(".QWidget {"
                                     "border: none;"
                                     "}")
        self.widget_coronal.setStyleSheet(".QWidget {"
                                          "border: none;"
                                          "}")
        self.widget_sagittal.setStyleSheet(".QWidget {"
                                           "border: none;"
                                           "}")
        self.widget_transverse.setStyleSheet(".QWidget {"
                                             "border: none;"
                                             "}")
        if quadrant == 1:
            self.widget_coronal.setStyleSheet(".QWidget {"
                                              "border: 1px solid tomato;"
                                              "}")
        elif quadrant == 2:
            self.widget_3d.setStyleSheet(".QWidget {"
                                         "border: 1px solid tomato;"
                                         "}")
        elif quadrant == 3:
            self.widget_sagittal.setStyleSheet(".QWidget {"
                                               "border: 1px solid tomato;"
                                               "}")
        elif quadrant == 4:
            self.widget_transverse.setStyleSheet(".QWidget {"
                                                 "border: 1px solid tomato;"
                                                 "}")

    @Slot()
    def handle_file_open(self):
        self.file_open_dialog.exec()
